/***************************************************************************
 *cr                                                                       
 *cr            (C) Copyright 1995 The Board of Trustees of the           
 *cr                        University of Illinois                       
 *cr                         All Rights Reserved                        
 *cr                                                                   
 ***************************************************************************/

/***************************************************************************
 * RCS INFORMATION:
 *
 *      $RCSfile: CmdUtil.C,v $
 *      $Author: billh $        $Locker:  $                $State: Exp $
 *      $Revision: 1.3 $      $Date: 1995/05/13 01:35:06 $
 *
 ***************************************************************************
 * DESCRIPTION:
 * 
 * Command objects for doing various utilities, such as executing a shell,
 * displaying help, or quitting
 *
 ***************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "CmdUtil.h"
#include "UIList.h"
#include "UIText.h"
#include "DisplayDevice.h"
#include "CommandQueue.h"
#include "Global.h"
#include "utilities.h"

// The following uses the Cmdtypes HELP, RESET, DEBUG,
// QUIT, WAIT, ECHO, LOGWRITE, LOGREAD from the Command class

////////////////////////////////////////////////////////////////////
///////////////////////  text processors
////////////////////////////////////////////////////////////////////

// text callback routine for 'quit'; return TRUE if an error occurs.
int text_cmd_quit(int argc, char **argv, CommandQueue *cmdQueue, int id) {
  if(argc == 1) {
    cmdQueue->append(new CmdQuit(FALSE,id));

  } else if(argc == 2) {
    if(!strupncmp(argv[1], "confirm", CMDLEN))
      cmdQueue->append(new CmdQuit(TRUE, id));
    else if(!strupncmp(argv[1], "now", CMDLEN))
      cmdQueue->append(new CmdQuit(FALSE, id));
    else
      return TRUE;

  } else
    return TRUE;

  // if here, no problem with command
  return FALSE;
}

// text callback routine for 'help'; return TRUE if an error occurs.
int text_cmd_help(int argc, char **argv, CommandQueue *cmdQueue, int id) {
  if(argc == 1) {
    cmdQueue->append(new CmdHelp(NULL, id));

  } else if(argc == 2) {
    cmdQueue->append(new CmdHelp(argv[1], id));

  } else
    return TRUE;

  // if here, no problem with command
  return FALSE;
}

// text callback routine for 'wait'; return TRUE if an error occurs.
int text_cmd_wait(int argc, char **argv, CommandQueue *cmdQueue, int id) {
  if(argc == 2) {
    cmdQueue->append(new CmdWait(atof(argv[1]), id));

  } else
    return TRUE;

  // if here, no problem with command
  return FALSE;
}

// text callback routine for 'echo'; return TRUE if an error occurs.
int text_cmd_echo(int argc, char **argv, CommandQueue *cmdQueue, int id) {
  if(argc == 2) {
    cmdQueue->append(new CmdEcho(argv[1], id));

  } else
    return TRUE;

  // if here, no problem with command
  return FALSE;
}

// text callback routine for 'debug'; return TRUE if an error occurs.
int text_cmd_debug(int argc, char **argv, CommandQueue *cmdQueue, int id) {
  if(argc == 2) {
    cmdQueue->append(new CmdDebug(CmdDebug::TOGGLE,
		!strupncmp(argv[1],"on",CMDLEN), id));

  } else if(argc == 3) {
    if(!strupncmp(argv[1], "level",CMDLEN))
      cmdQueue->append(new CmdDebug(CmdDebug::LEVEL, atoi(argv[2]), id));
    else
      return TRUE;

  } else
    return TRUE;

  // if here, no problem with command
  return FALSE;
}

// text callback routine for 'log'; return TRUE if an error occurs.
int text_cmd_log(int argc, char **argv, CommandQueue *cmdQueue, int id) {
  if(argc == 2) {
    char *logfile;
    logfile = ((!strupncmp(argv[1],"off",CMDLEN)) ? NULL : argv[1]);
    cmdQueue->append(new CmdLogWrite(logfile, id));

  } else
    return TRUE;

  // if here, no problem with command
  return FALSE;
}

// text callback routine for 'play'; return TRUE if an error occurs.
int text_cmd_play(int argc, char **argv, CommandQueue *cmdQueue, int id) {
  if(argc == 2) {
    cmdQueue->append(new CmdLogRead(argv[1], id));

  } else
    return TRUE;

  // if here, no problem with command
  return FALSE;
}


//////////// display help, in various formats
int CmdHelp::do_execute(void) {

  // create a string used to load the vmd help file
  char strbuf[256];
  strcpy(strbuf, dataPath);
  strcat(strbuf, "/vmd_help.html");
  if(word) {
    strcat(strbuf, "#");
    strcat(strbuf, word);
  }

  // try to start viewer ...
  return VMDreadHTML(strbuf);
}

void CmdHelp::create_text(void) {
  *cmdText << "help ";
  if(word)
    *cmdText << word;
  *cmdText << ends;
}

CmdHelp::CmdHelp(char *w, int newUIid) : Command(Command::HELP, newUIid) {
  word = (w != NULL) ? stringdup(w) : w;
}

CmdHelp::~CmdHelp(void) {
  if(word)
    delete [] word;
}


//////////// reset all the UI's
int CmdReset::do_execute(void) {
  int retval = TRUE;
  if(resetUI < 0) {
    if(display)
      display->reshape();
    uiList->reset_UI();
  } else if(resetUI >= 0 && resetUI < uiList->num()) {
    (uiList->item(resetUI))->reset();
  } else {
    retval = FALSE;
  }
  return retval;
}

CmdReset::CmdReset(int rUI, int newUIid) : Command(Command::RESET, newUIid) {
  resetUI = rUI;
}


//////////// exit the program, possibly asking for confirmation
int CmdQuit::do_execute(void) {
  int doexit = TRUE;
  if(askConfirm)
    // for now, does nothing
    ;
  if(doexit)
    VMDexit("Exiting normally", 0);

  return FALSE;		// only get here if confirm = no
}

void CmdQuit::create_text(void) {
  *cmdText << "quit " << (askConfirm ? "confirm" : "now") << ends;
}

CmdQuit::CmdQuit(int confirm, int newUIid) : Command(Command::QUIT, newUIid) {
  askConfirm = confirm;
}


//////////// set the debug level for the program
int CmdDebug::do_execute(void) {
  int retval = TRUE;
  if(action == TOGGLE) {
    msgDebug.on(newVal);
  } else if(action == LEVEL) {
    msgDebug.output_level(newVal);
  } else {
    retval = FALSE;
  }
  return retval;
}

void CmdDebug::create_text(void) {
  *cmdText << "debug ";
  if(action == TOGGLE) {
    *cmdText << (newVal ? "on" : "off");
  } else if(action == LEVEL) {
    *cmdText << "level " << newVal;
  }
  *cmdText << ends;
}

CmdDebug::CmdDebug(DebugAction actn, int nval, int newUIid)
	: Command(Command::DEBUG, newUIid) {
  action = actn;
  newVal = nval;
}


//////////// tell the text processor to wait an interval between commands
int CmdWait::do_execute(void) {
  uiText->wait(delay);
  return TRUE;
}

void CmdWait::create_text(void) {
  *cmdText << "wait " << delay << ends;
}

CmdWait::CmdWait(float d, int newUIid)
	: Command(Command::WAIT, newUIid) {
  delay = d;
}


//////////// toggle on/off whether to echo items to the console
int CmdEcho::do_execute(void) {
  if(onoff >= 0)
    uiText->echo(onoff);
  else
    msgErr << "You must specify 'on' or 'off' with echo." << sendmsg;
  return (onoff >= 0);
}

void CmdEcho::create_text(void) {
  if(onoff == 1 || onoff == 0)
    *cmdText << "echo " << (onoff == 1 ? "on" : "off");
  *cmdText << ends;
}

CmdEcho::CmdEcho(char *t, int newUIid) : Command(Command::ECHO, newUIid) {
  if(!strupncmp(t, "on", CMDLEN))
    onoff = 1;
  else if(!strupncmp(t, "off", CMDLEN))
    onoff = 0;
  else
    onoff = (-1);
}


//////////// toggle on/off whether to echo items to the console
int CmdLogWrite::do_execute(void) {
  if(logfile)
    commandQueue->log_on(logfile);
  else
    commandQueue->log_off();
  return TRUE;
}

CmdLogWrite::CmdLogWrite(char *t, int newUIid)
	: Command(Command::LOGWRITE, newUIid) {
  logfile = (t ? stringdup(t) : NULL);
}

CmdLogWrite::~CmdLogWrite(void) {
  if(logfile)
    delete [] logfile;
}


//////////// start playing back a log file/script
int CmdLogRead::do_execute(void) {
  uiText->read_from_file(logfile);
  return TRUE;
}

CmdLogRead::CmdLogRead(char *t, int newUIid)
	: Command(Command::LOGREAD, newUIid) {
  logfile = stringdup(t);
}

CmdLogRead::~CmdLogRead(void) {
  delete [] logfile;
}


/* REVISION HISTORY:********************************************************
 *
 * $Log: CmdUtil.C,v $
 * Revision 1.3  1995/05/13  01:35:06  billh
 * Help is now displayed by launching an HTML viewer (Mosaic by default)
 * which will display the file /usr/local/lib/vmd/vmd_help.html
 *
 * Revision 1.2  95/05/11  22:06:31  billh
 * Added 'create_text' function to create text representation "on demand".
 * Added text callback functions which are added by UIText, and which are
 * called when a text command is entered and the first word is recognized.
 * 
 * Revision 1.3  1994/10/05  06:58:16  billh
 * Converted to use a ResizeArray instead of a DLinkList in the command queue
 * and ui list.
 *
 * Revision 1.2  1994/10/04  20:29:22  billh
 * Changes to get to compile on HP's ... not there yet,though.
 *
 * Revision 1.1  1994/08/24  03:10:37  billh
 * Initial revision
 *
 ***************************************************************************/
