/***************************************************************************
 *cr                                                                       
 *cr            (C) Copyright 1995 The Board of Trustees of the           
 *cr                        University of Illinois                       
 *cr                         All Rights Reserved                        
 *cr                                                                   
 ***************************************************************************/

/***************************************************************************
 * RCS INFORMATION:
 *
 *	$RCSfile: Stage.h,v $
 *	$Author: billh $	$Locker:  $		$State: Exp $
 *	$Revision: 1.7 $	$Date: 1995/05/11 23:57:09 $
 *
 ***************************************************************************
 * DESCRIPTION:
 *
 * A Stage is a displayable object that acts as floor for other objects.
 * It is intended to be a point of reference.
 *
 ***************************************************************************/
#ifndef STAGE_H
#define STAGE_H

#include "Displayable.h"
#include "DispCmds.h"

#define STAGE_PANELS	8

class Stage : public Displayable3D {

public:
  // locations for the stage
  enum StagePos { NO_STAGE = 0, STAGE_ORIGIN, STAGE_LOWER, STAGE_UPPER,
  	STAGE_LEFT, STAGE_RIGHT, STAGE_BEHIND, STAGEPOS_TOTAL };

private:
  // corners defining the stage
  float corner1[3], corner2[3], corner3[3], corner4[3];
  float xw, zw;

  // number of panels each side is divided into, and width of panel
  int Panels;
  float inc;

  // current stage position
  int stagePos;
  
  // do we need an update
  int need_update;

  // useful display command objects
  DispCmdMaterials cmdMaterials;
  DispCmdColorIndex cmdColor;
  DispCmdSquare cmdSquare;

  // integer for color category with the colors to use.  If < 0, use
  // default colors
  int colorCat;

  // regenerate the command listo
  void create_cmdlist(void);

protected:
  // do action when a new color list is provided
  virtual void do_use_colors(void);

  // do action due to the fact that a color for the given ColorList for
  // the specified category has changed
  virtual void do_color_changed(ColorList *, int);

public:
  // constructor: the Scene to register this with
  Stage(Scene *);

  // set stage display mode; return success
  int location(int);

  // return stage display mode
  int location(void) { return stagePos; }

  // return descripton of location
  char *loc_description(int);

  // return total number of locations
  int locations(void) { return STAGEPOS_TOTAL; }

  // get/set number of panels (must be >= 1)
  int panels(void) { return Panels; }
  int panels(int newp) {
    int retval = FALSE;
    if(newp >= 1 && newp <= 10 && newp != Panels) {
      Panels = newp;
      inc = 2.0 / (float)Panels;
      need_update = TRUE;
      retval = TRUE;
    }
    return retval;
  }

  //
  // public virtual routines
  //
  
  // prepare for drawing ... do any updates needed right before draw.
  virtual void prepare(DisplayDevice *);

};

#endif

/* REVISION HISTORY:********************************************************
 *
 * $Log: Stage.h,v $
 * Revision 1.7  1995/05/11  23:57:09  billh
 * Moved log messages to end of file.
 *
 * Revision 1.6  95/03/24  18:51:53  billh
 * Added copyright notice to top of file; made sure all virtual routines
 * are defined in the .C file, not in the .h file.
 * 
 * Revision 1.5  1994/12/06  08:24:25  billh
 * Added ability to create color categories, due to being now derived from
 * ColorUser.
 * Added routines to have a new color list specified, and to check for when
 * a color in a relevant category is changed.
 *
 * Revision 1.4  94/09/29  06:51:39  billh
 * Changed location() routine to be inline.
 * 
 * Revision 1.3  1994/09/24  20:09:27  billh
 * Now contain single display commands which are used many times,
 * instead of re-creating and destroying disp command objects.
 *
 * Revision 1.2  94/09/05  22:49:43  billh
 * Completed initial code, now can actually draw something at different
 * locations.
 * 
 * Revision 1.1  1994/08/24  03:10:37  billh
 * Initial revision
 *
 ***************************************************************************/
